% Exemplo de controle LQG

% Seja a planta discreta
Phi    = [0.7 -0.3 ; 0.1 0.5];
Gamma  = eye(2);
C      = eye(2);
D      = zeros(2,2);

Lambda = eye(2);
W = [0.1 0; 0 0.2];
V = [1.0 0; 0 2.0];

T  = 0.1;
S  = ss(Phi,Gamma,C,D,T);
Sx = ss(Phi,Gamma,eye(2),D,T);

% Regulador linear quadrtico
Q = [2 0; 0 1];
R = [100 0; 0 100];
[K,X,zc] = dlqr(Phi,Gamma,Q,R);


% Filtro de Kalman

% Sistema aumentado para projetar o filtro (Veja '>> doc kalman' no Matlab)
H = zeros(2,2);
Saug = ss(Phi,[Gamma Lambda],C,[D H],T); 

[KF,L,P]=kalman(Saug,W,V); 


% Simulao
xo = [10 -10];

sim('ex_lqg_sim');


% Grficos
figure(1);
subplot(211)
h1 = plot(tsim,ysim(:,1));
ColOrder = get(gca,'ColorOrder');
set(h1,'linewidth',2,'Color',ColOrder(1,:))
hold on
h2 = plot(tsim,xsim(:,1));
set(h2,'linewidth',2,'Color',ColOrder(2,:))
h3 = plot(tsim,xkf(:,1));
set(h3,'linewidth',2,'Color',ColOrder(3,:))
hold off
grid on
%axis([0 5 -3 6])
xlabel('t (s)');
ylabel('(componente 1)')
title('Sistema em malha fechada (Controle LQG)')
legend([h1 h2 h3],'sada medida','estado efetivo (no medido)','estado estimado','location','northeast')

subplot(212)
h1 = plot(tsim,ysim(:,2));
ColOrder = get(gca,'ColorOrder');
set(h1,'linewidth',2,'Color',ColOrder(1,:))
hold on
h2 = plot(tsim,xsim(:,2));
set(h2,'linewidth',2,'Color',ColOrder(2,:))
h3 = plot(tsim,xkf(:,2));
set(h3,'linewidth',2,'Color',ColOrder(3,:))
hold off
grid on
%axis([0 5 -3 6])
xlabel('t (s)');
ylabel('(componente 2)')
legend([h1 h2 h3],'sada medida','estado efetivo (no medido)','estado estimado','location','southeast')